@@ -1,5 +1,6 @@
-      subroutine lokern_s(t,x, tt,y, n,m,nue,kord, hetero,isrand,
-     .     inputb,m1,tl,tu,s,sig,wn,w1, wm,ban, trace)
+c     called from R's .lokerns() in ../R/lokerns.R
+      subroutine lokern_s(t,x, tt,y, n,m,nue,kord, hetero,
+     .     israndI,inputbI, m1,tl,tu,s,sig, wn,w1, wm,ban, trace)
 c----------------------------------------------------------------------*
 c-----------------------------------------------------------------------
 c       Short-version: January 1997
@@ -13,6 +14,8 @@ c       (nue,kord) = (0,2), (0,4), (1,3) or (2,4).
 c-----------------------------------------------------------------------
 c  used subroutines: constV, resest, kernel with further subroutines
 c-----------------------------------------------------------------------
+      implicit none
+c
 c Args
       integer n, m, nue,kord
       double precision t(n), x(n),
@@ -20,35 +23,37 @@ c Args
      +     tl, tu,
      +     s(0:n), ! mid point values for convolution kernel
      +     sig
-       integer hetero, isrand, inputb
-c      logical hetero, isrand, inputb
-c			      inputb (was "smo", now same as in R):
-c if TRUE, do not compute bandwidths but use ban(.)
-      integer m1, trace
+      integer hetero, israndI, inputbI, m1
       double precision wn(0:n,5), w1(m1,3), wm(m), ban(m)
+      integer trace
+
 c Var
+c if TRUE, do not compute bandwidths but use ban(.)
       logical inputs, needsrt
-      integer nyg, i,ii,iil,itt,il,iu,itende,it, j, kk,kk2, nn
-     1     , kordv, nuev, nyl ! <- lokern extra
+      integer i,ii,iil,itt,il,iu,itende,it, j, kk,kk2, nn
+     1     , kordv, nuev ! <- lokern extra
       double precision bias(2,0:2), vark(2,0:2), fak2(2:4),
      1     rvar, s0,sn, b2,bmin,bmax, bres,bs, alpha,ex,exs,exsvi,
      2     r2,snr,osig, vi,ssi,const,fac, q,tll,tuu, xi,xmy2
-     3     , b, bvar, g1,g2,dist, wstep, xh,xxh ! <- lokern extra
-c-
-c-------- 1. initialisations
+     3   , b, bvar, g1,g2,dist, wstep, xh,xxh ! <- lokern extra
+      logical isrand, inputb
+c		      inputb (was "smo", now same as in R):
+
+c-------- 1. initialisations ('data' *first*) ----------
+
       data bias/.2, .04762, .4286, .1515, 1.33, .6293/
       data vark/.6,  1.250, 2.143, 11.93, 35.0, 381.6/
       data fak2/4.,36.,576./
-      nyg=0
       inputs = .false.
-c r2:  used in phase 17, but only defined in phase 9 if(hetero & sig <= 0)
+      isrand = (israndI .ne. 0)
+      inputb = (inputbI .ne. 0)
+c     r2:  used in phase 17, but only defined in phase 9 if(hetero & sig <= 0)
       r2=0.
-      nyl=1
 c Stop for invalid inputs (impossible when called from R's lokerns())
 
 c     0 <= nue <= 4;  nue <= 2 if(! inputb)
       if(nue.gt.4 .or. nue.lt.0) call rexit("nue must be in 0..4")
-      if(nue.gt.2 .and. inputb .ne. 0)
+      if(nue.gt.2 .and. .not. inputb)
      +     call rexit("nue must be in 0..2 if not 'inputb'")
       if(n .le. 2) call rexit("n <= 2")
       if(m .lt. 1) call rexit("m < 1")
@@ -57,16 +62,18 @@ c     0 <= nue <= 4;  nue <= 2 if(! inputb)
 c     kord - nue must be even :
       kk=(kord-nue)/2
       if(2*kk + nue .ne. kord)        kord=nue+2
-      if(kord.gt.4 .and. inputb .eq. 0) kord=nue+2
+      if(kord.gt.4 .and. .not.inputb) kord=nue+2
       if(kord.gt.6 .or. kord.le.nue)  kord=nue+2
 
+      if(trace .gt. 0) then
+         call monit0(1, n, m, nue, kord, inputbI, israndI, ban, trace)
+      end if
+
       rvar=sig
       itende = -1
       il=1
       iu=n
-
-      if(trace .gt. 0) call monit0(1, n, m, nue, kord,
-     +     inputb, isrand, ban, trace)
+      b = -1. ! for local bandwidths, sometimes need  b := mean(ban[]) later
 
 c-------- 2. computation of s-sequence
       if(trace .gt. 0) call monit1(2, trace)
@@ -79,9 +86,12 @@ c-------- 2. computation of s-sequence
          end do
          s(0)=s0
          s(n)=sn
-         if(inputb .ne. 0 .and. isrand .eq. 0) goto 230
+         if(inputb .and. .not.isrand) then
+            if(trace .gt. 0) call monit2ib(trace)
+            goto 230
+         end if
       else
-         if(inputb .ne. 0) goto 230
+         if(inputb) goto 230
       end if
 c-
 c-------- 3. computation of minimal, maximal allowed global bandwidth
@@ -92,7 +102,9 @@ c-
 c-------- 4. compute tl,tu
       if(trace .gt. 0) call monit1(4, trace)
       itt=0
-40    if (tu.le.tl) then
+ 40   continue
+      if(trace .gt. 0) call monit40(itt, il,iu, tl,tu, trace)
+      if (tu.le.tl) then
         tl=.933*s0+.067*sn
         tu=.067*s0+.933*sn
         itt=itt+1
@@ -162,7 +174,7 @@ c-
 c-------- 9. estimating variance and smoothed pseudoresiduals
       if(trace .gt. 0) call monit1(9, trace)
       rvar=sig ! to become old 'sig'
-      if(hetero .ne. 0) then
+      if(hetero .eq. 1) then
         call resest(t,x,n,wn(1,2),snr,sig)
         bres=max(bmin,.2*dble(nn)**(-.2)*(s(iu)-s(il-1)))
         do i=1,n
@@ -170,9 +182,9 @@ c-------- 9. estimating variance and smoothed pseudoresiduals
            wn(i,2)=wn(i,2)*wn(i,2)
         end do
 c     smooth  (t[i], r[i]^2) , r[]= (leave-one-out interpol.) residual from reset
-        call kernel(t,wn(1,2),n,bres,0,kk2,nyg,s,
-     .       wn(1,3),n,wn(1,4), trace)
-cc ?? FIXME     ^         ^  ./glkerns.f has 'il' here instead of '1'
+        call kernel(t,wn(1,2),n, bres, 0, kk2, 0, ! <- ny=0 : global bandwidth
+     .       s, wn(1,3),n,wn(1,4), trace)
+cc ?? FIXME        ^         ^  ./glkerns.f has 'il' here instead of '1'
       else !-- not hetero
          if(sig .le. 0.) then
             call resest(t(il),x(il),nn,wn(il,2),r2,sig)
@@ -186,10 +198,11 @@ c-------- 10. [LOOP:] estimate/compute integral constant
       do i=il,iu
          vi=vi+ wn(i,1)*n*(s(i)-s(i-1))**2 * wn(i,4)
       end do
+      if(trace .gt. 0) call monit1c(il,iu, vi, trace)
 c-
 c-------- 11. refinement of s-sequence for random design
-      if(trace .ge. 2) call monit1(11, trace)
-      if(inputs .and. isrand .ne. 0) then
+      if(trace .gt. 0) call monit1(11, trace)
+      if(inputs .and. isrand) then
         do i=0,n
           wn(i,5)=dble(i)/dble(n+1)
           wn(i,2)=(dble(i)+.5)/dble(n+1)
@@ -198,28 +211,30 @@ c-------- 11. refinement of s-sequence for random design
         exs= -dble(3*kord+1) / dble(6*kord+3)
         exsvi=dble(kord)     / dble(6*kord+3)
         bs=0.1*(vi/(sn-s0)**2)**exsvi * dble(n)**exs
-        call kernel(wn(1,5),t,n,bs,0,2,nyg,wn(0,3),wn(0,2),n+1,s(0),
-     .       trace)
+        call kernel(wn(1,5),t,n,bs,0,2, 0, ! <- ny=0 : global bandwidth
+     .       wn(0,3),wn(0,2), n+1, s(0), trace)
+c    MM vvvvvv shouldn't this really happen *after* 111  ??
         vi=0.0
 111     needsrt=.false.
         do i=1,n
            vi=vi+ wn(i,1)*n*(s(i)-s(i-1))**2 * wn(i,4)
-           if(s(i).lt.s(i-1)) then
+           if(s(i).lt.s(i-1)) then ! swap them
               ssi=s(i-1)
               s(i-1)=s(i)
               s(i)=ssi
               needsrt=.true.
            end if
+           if(needsrt .and. trace .gt. 0) call monit111(i, s(i-1),ssi)
         end do
         if(needsrt) goto 111
-        if(inputb .ne. 0) goto 230
+        if(inputb) goto 230
       end if
       b=bmin*2.
 c-
 c-------- 12. compute inflation constant and exponent and loop of iterations
-      if(trace .ge. 2) call monit1(12, trace)
+      if(trace .gt. 0) call monit1(12, trace)
       const=dble(2*nue+1)*fak2(kord)*vark(kk,nue)*vi
-     .       /(dble(2*kord-2*nue)*bias(kk,nue)**2*dble(n))
+     .       / (dble(2*kord-2*nue) * bias(kk,nue)**2 * dble(n))
       fac=1.1*(1.+(nue/10.)+0.05*(kord-nue-2.))
      .       * dble(n)**(2./dble((2*kord+1)*(2*kord+3)))
 
@@ -227,12 +242,14 @@ c     itende=1+2*kord+kord*(2*kord+1)
       itende = (1 + 2*kord) * (1 + kord)
 c     ^^^^^^  *fixed* number of iterations ( <== theory !)
 
+c     ================================ iterations ===============
       do it=1,itende
 c-
 c-------- 13. estimate derivative of order kord in iterations
         if(trace .ge. 3) call monit1(13, trace)
         b2 = min(bmax, max(b*fac, bmin/dble(kord-1)*dble(kord+1)))
-        call kernel(t,x,n,b2,kord,kord+2,nyg,s,w1(1,1),m1,w1(1,3),trace)
+        call kernel(t,x,n,b2,kord,kord+2, 0, ! <- ny=0: global bandwidth
+     .              s, w1(1,1), m1, w1(1,3), trace-2)
 c-
 c-------- 14. estimate integralfunctional in iterations
         if(trace .ge. 3) call monit1(14, trace)
@@ -246,14 +263,16 @@ c-------- 15. finish of iterations
         if(trace .ge. 3) call monit1(15, trace)
         b = min(bmax, max(bmin, (const/xmy2)**ex))
       end do
+c     ====== end iterations -------------------------------------
 
 c-------- 16  compute smoothed function with global plug-in bandwidth
       if(trace .ge. 2) call monit1(16, trace)
-      call kernel(t,x,n,b,nue,kord,nyg,s,tt,m,y, trace)
+      call kernel(t,x,n,b,nue,kord, 0, ! <- ny=0: global bandwidth
+     .            s,tt, m,y, trace-1)
 c-
 c-------- 17. variance check
       if(trace .ge. 2) call monit1(17, trace)
-      if(hetero .ne. 0) sig=rvar
+      if(hetero .eq. 1) sig=rvar
       if(sig.eq.rvar .or. r2.lt.0.88 .or. nue.gt.0) goto 180
       ii=0
       iil=0
@@ -318,28 +337,28 @@ c-------- 20. estimate/compute integral constant vi locally
          wm(j)=tt(j)
          if(tt(j).lt.s(0)+g1) then
             dist=((tt(j)-g1-s(0))/g1)**2
-            ban(j)=bvar*(1.0+1.0*dist)
-            ban(j)=min(ban(j),bmax)
+            ban(j)=min(bmax, bvar*(1.0+1.0*dist))
             wm(j)=tt(j)+.5*dist*g1
          else if(tt(j).gt.s(n)-g1) then
             dist=((tt(j)-s(n)+g1)/g1)**2
-            ban(j)=bvar*(1.0+1.0*dist)
-            ban(j)=min(ban(j),bmax)
+            ban(j)=min(bmax, bvar*(1.0+1.0*dist))
             wm(j)=tt(j)-.5*dist*g1
          end if
       end do
-      call kernel(t,wn(1,4),n,bvar,nuev,kordv,nyl,s,wm,m,ban,trace)
+      call kernel(t,wn(1,4),n,bvar,nuev,kordv, 1, ! ny=1 : *local* bandwidths
+     .            s,wm,m,ban,trace)
 c-
 c-------- 21. estimation of kord-th derivative locally
       if(trace .gt. 0) call monit1(21, trace)
       wstep=(tt(m)-tt(1))/dble(m1-2)
       do j=2,m1
-         w1(j,2)=tt(1)+dble(j-2)*wstep
+         w1(j,2)=tt(1)+dble(j-2  )*wstep
          w1(j,1)=tt(1)+dble(j-1.5)*wstep
       end do
       w1(1,1)=tt(1)+.5*wstep
 
-      call kernel(t,x,n,g1,kord,kord+2,nyg,s,w1(2,2),m1-1,w1(2,3),trace)
+      call kernel(t,x,n,g1,kord,kord+2, 0, ! ny=0 : *global* bandwidths
+     .            s, w1(2,2), m1-1, w1(2,3), trace)
 
       do j=2,m1
         w1(j,3)=w1(j,3)*w1(j,3)
@@ -347,16 +366,14 @@ c-------- 21. estimation of kord-th derivative locally
       do j=1,m
          y(j)=g2
          if(tt(j).lt.s(0)+g1) then
-            y(j)=g2*(1.0+1.0*((tt(j)-g1-s(0))/g1)**2)
-            y(j)=min(y(j),bmax)
+            y(j)=min(bmax, g2*(1.0+1.0*((tt(j)-g1-s(0))/g1)**2))
          else if(tt(j).gt.s(n)-g1) then
-            y(j)=g2*(1.0+1.0*((tt(j)-s(n)+g1)/g1)**2)
-            y(j)=min(y(j),bmax)
+            y(j)=min(bmax, g2*(1.0+1.0*((tt(j)-s(n)+g1)/g1)**2))
          end if
       end do
 
-      call kernp(w1(2,2),w1(2,3),m1-1,g2,nuev,kordv,nyl, w1(1,1),wm,m,y,
-     .     trace)
+      call kernp(w1(2,2),w1(2,3),m1-1,g2,nuev,kordv, 1,! ny=1: *local* bandwidths
+     .           w1(1,1), wm, m,y, trace)
 c-
 c-------- 22. finish
 c-what for? irnd=1-irnd
@@ -373,13 +390,21 @@ c-what for? irnd=1-irnd
          end if
       end do
 c-
-c-------- 23. compute smoothed function with local plug-in bandwidth
+c-------- 23. compute smoothed function with *local* plug-in bandwidth
  230  continue
       if(trace .gt. 0) call monit1(23, trace)
       do j=1,m
         y(j)=ban(j)
       end do
-      call kernel(t,x,n,b,nue,kord,nyl,s,tt,m,y, trace)
+      if(b .eq. -1.) then ! compute b := mean(ban[])
+         b=0.
+         do j=1,m
+            b=b + ban(j)
+         end do
+         b=b/m
+      end if
+      call kernel(t,x,n,b,nue,kord, 1,! ny=1: *local* bandwidths
+     .            s,tt, m,y, trace)
 c-  return #{iter} (iff aplicable)
       m1=itende
 
